package vieon.reactlibrary.extensions.auto_start

import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.content.pm.ApplicationInfo
import android.os.Build
import com.facebook.react.bridge.Promise

class AutoStartPermissionHelper private constructor() {

  /***
   * Xiaomi
   */
  private val BRAND_XIAOMI = "xiaomi"
  private val PACKAGE_XIAOMI_MAIN = "com.miui.securitycenter"
  private val PACKAGE_XIAOMI_COMPONENT = "com.miui.permcenter.autostart.AutoStartManagementActivity"

  /***
   * Letv
   */
  private val BRAND_LETV = "letv"
  private val PACKAGE_LETV_MAIN = "com.letv.android.letvsafe"
  private val PACKAGE_LETV_COMPONENT = "com.letv.android.letvsafe.AutobootManageActivity"

  /***
   * ASUS ROG
   */
  private val BRAND_ASUS = "asus"
  private val PACKAGE_ASUS_MAIN = "com.asus.mobilemanager"
  private val PACKAGE_ASUS_COMPONENT = "com.asus.mobilemanager.powersaver.PowerSaverSettings"

  /***
   * Honor
   */
  private val BRAND_HONOR = "honor"
  private val PACKAGE_HONOR_MAIN = "com.huawei.systemmanager"
  private val PACKAGE_HONOR_COMPONENT = "com.huawei.systemmanager.optimize.process.ProtectActivity"

  /**
   * Oppo
   */
  private val BRAND_OPPO = "oppo"
  private val PACKAGE_OPPO_MAIN = "com.coloros.safecenter"
  private val PACKAGE_OPPO_FALLBACK = "com.oppo.safe"
  private val PACKAGE_OPPO_COMPONENT = "com.coloros.safecenter.permission.startup.StartupAppListActivity"
  private val PACKAGE_OPPO_COMPONENT_FALLBACK = "com.oppo.safe.permission.startup.StartupAppListActivity"
  private val PACKAGE_OPPO_COMPONENT_FALLBACK_A = "com.coloros.safecenter.startupapp.StartupAppListActivity"

  /**
   * Vivo
   */

  private val BRAND_VIVO = "vivo"
  private val PACKAGE_VIVO_MAIN = "com.iqoo.secure"
  private val PACKAGE_VIVO_FALLBACK = "com.vivo.permissionmanager"
  private val PACKAGE_VIVO_COMPONENT = "com.iqoo.secure.ui.phoneoptimize.AddWhiteListActivity"
  private val PACKAGE_VIVO_COMPONENT_FALLBACK = "com.vivo.permissionmanager.activity.BgStartUpManagerActivity"
  private val PACKAGE_VIVO_COMPONENT_FALLBACK_A = "com.iqoo.secure.ui.phoneoptimize.BgStartUpManager"

  /**
   * Nokia
   */

  private val BRAND_NOKIA = "nokia"
  private val PACKAGE_NOKIA_MAIN = "com.evenwell.powersaving.g3"
  private val PACKAGE_NOKIA_COMPONENT = "com.evenwell.powersaving.g3.exception.PowerSaverExceptionActivity"

  fun getAutoStartPermission(context: Context, promise: Promise) {

    val build_info = Build.BRAND.toLowerCase()

    when (build_info) {

      BRAND_ASUS -> autoStartAsus(context,promise)

      BRAND_XIAOMI -> autoStartXiaomi(context,promise)

      BRAND_LETV -> autoStartLetv(context,promise)

      BRAND_HONOR -> autoStartHonor(context,promise)

      BRAND_OPPO -> autoStartOppo(context,promise)

      BRAND_VIVO -> autoStartVivo(context,promise)

      BRAND_NOKIA -> autoStartNokia(context,promise)

      else -> promise.reject(Throwable("No match MANUFACTURER"))
    }

  }

  private fun autoStartXiaomi(context: Context,
      promise: Promise) {
    if (isPackageExists(context, PACKAGE_XIAOMI_MAIN)) {
      try {
        startIntent(context, PACKAGE_XIAOMI_MAIN, PACKAGE_XIAOMI_COMPONENT)
        promise.resolve(true)
      } catch (e: Exception) {
        e.printStackTrace()
        promise.reject(e)
      }

    }else{
      promise.reject(Throwable("No AutoStart manager found"))
    }
  }

  private fun autoStartAsus(context: Context,
      promise: Promise) {
    if (isPackageExists(context, PACKAGE_ASUS_MAIN)) {
      try {
        startIntent(context, PACKAGE_ASUS_MAIN, PACKAGE_ASUS_COMPONENT)
        promise.resolve(true)
      } catch (e: Exception) {
        e.printStackTrace()
        promise.reject(e)
      }

    }else{
      promise.reject(Throwable("No AutoStart manager found"))
    }
  }

  private fun autoStartLetv(context: Context,
      promise: Promise) {
    if (isPackageExists(context, PACKAGE_LETV_MAIN)) {
      try {
        startIntent(context, PACKAGE_LETV_MAIN, PACKAGE_LETV_COMPONENT)
        promise.resolve(true)
      } catch (e: Exception) {
        e.printStackTrace()
        promise.reject(e)
      }

    }else{
      promise.reject(Throwable("No AutoStart manager found"))
    }
  }

  private fun autoStartHonor(context: Context,
      promise: Promise) {
    if (isPackageExists(context, PACKAGE_HONOR_MAIN)) {
      try {
        startIntent(context, PACKAGE_HONOR_MAIN, PACKAGE_HONOR_COMPONENT)
        promise.resolve(true)
      } catch (e: Exception) {
        e.printStackTrace()
        promise.reject(e)
      }

    }else{
      promise.reject(Throwable("No AutoStart manager found"))
    }
  }

  private fun autoStartOppo(context: Context,
      promise: Promise) {
    if (Build.MANUFACTURER == "OPPO") {
      try {
        startIntent(context,"com.coloros.safecenter",
            "com.coloros.safecenter.permission.startup.FakeActivity")
        promise.resolve(true)
      } catch (e: Exception) {
        try {
          startIntent(context,"com.coloros.safecenter",
              "com.coloros.safecenter.permission.startupapp.StartupAppListActivity")
          promise.resolve(true)
        } catch (e1: Exception) {
          try {
            startIntent(context,"com.coloros.safecenter",
                "com.coloros.safecenter.permission.startupmanager.StartupAppListActivity")
            promise.resolve(true)
          } catch (e2: Exception) {
            try {
              startIntent(context,"com.coloros.safe",
                  "com.coloros.safe.permission.startup.StartupAppListActivity")
              promise.resolve(true)
            } catch (e3: Exception) {
              try {
                startIntent(context,"com.coloros.safe",
                    "com.coloros.safe.permission.startupapp.StartupAppListActivity")
                promise.resolve(true)
              } catch (e4: Exception) {
                try {
                  startIntent(context,"com.coloros.safe",
                      "com.coloros.safe.permission.startupmanager.StartupAppListActivity")
                  promise.resolve(true)
                } catch (e5: Exception) {
                  try {
                    startIntent(context,"com.coloros.safecenter",
                        "com.coloros.safecenter.permission.startsettings")
                    promise.resolve(true)
                  } catch (e6: Exception) {
                    try {
                      startIntent(context,"com.coloros.safecenter",
                          "com.coloros.safecenter.permission.startupapp.startupmanager")
                      promise.resolve(true)
                    } catch (e7: Exception) {
                      try {
                        startIntent(context,"com.coloros.safecenter",
                            "com.coloros.safecenter.permission.startupmanager.startupActivity")
                        promise.resolve(true)
                      } catch (e8: Exception) {
                        try {
                          startIntent(context,"com.coloros.safecenter",
                              "com.coloros.safecenter.permission.startup.startupapp.startupmanager")
                          promise.resolve(true)
                        } catch (e9: Exception) {
                          try {
                            startIntent(context,"com.coloros.safecenter",
                                "com.coloros.privacypermissionsentry.PermissionTopActivity.Startupmanager")
                            promise.resolve(true)
                          } catch (e10: Exception) {
                            try {
                              startIntent(context,"com.coloros.safecenter",
                                  "com.coloros.privacypermissionsentry.PermissionTopActivity")
                              promise.resolve(true)
                            } catch (e11: Exception) {
                              try {
                                startIntent(context,"com.coloros.safecenter",
                                    "com.coloros.safecenter.FakeActivity")
                                promise.resolve(true)
                              } catch (e12: Exception) {
                                e12.printStackTrace()
                                promise.reject(e)
                              }

                            }

                          }

                        }

                      }

                    }

                  }

                }

              }

            }

          }

        }

      }

    }
  }

  private fun autoStartVivo(context: Context,
      promise: Promise) {
    if (isPackageExists(context, PACKAGE_VIVO_MAIN) || isPackageExists(context, PACKAGE_VIVO_FALLBACK)) {
      try {
        startIntent(context, PACKAGE_VIVO_MAIN, PACKAGE_VIVO_COMPONENT)
        promise.resolve(true)
      } catch (e: Exception) {
        e.printStackTrace()
        try {
          startIntent(context, PACKAGE_VIVO_FALLBACK, PACKAGE_VIVO_COMPONENT_FALLBACK)
          promise.resolve(true)
        } catch (ex: Exception) {
          ex.printStackTrace()
          try {
            startIntent(context, PACKAGE_VIVO_MAIN, PACKAGE_VIVO_COMPONENT_FALLBACK_A)
            promise.resolve(true)
          } catch (exx: Exception) {
            exx.printStackTrace()
            promise.reject(exx)
          }

        }

      }

    }else{
      promise.reject(Throwable("No AutoStart manager found"))
    }
  }

  private fun autoStartNokia(context: Context,
      promise: Promise) {
    if (isPackageExists(context, PACKAGE_NOKIA_MAIN)) {
      try {
        startIntent(context, PACKAGE_NOKIA_MAIN, PACKAGE_NOKIA_COMPONENT)
        promise.resolve(true)
      } catch (e: Exception) {
        e.printStackTrace()
        promise.reject(e)
      }
    }else{
      promise.reject(Throwable("No AutoStart manager found"))
    }
  }

  @Throws(Exception::class)
  private fun startIntent(context: Context, packageName: String, componentName: String) {
    try {
      val intent = Intent()
      intent.component = ComponentName(packageName, componentName)
      context.startActivity(intent)
    } catch (exception: Exception) {
      exception.printStackTrace()
      throw exception
    }

  }

  private fun isPackageExists(context: Context, targetPackage: String): Boolean {
    val packages: List<ApplicationInfo>
    val pm = context.packageManager
    packages = pm.getInstalledApplications(0)
    for (packageInfo in packages) {
      if (packageInfo.packageName == targetPackage) {
        return true
      }
    }
    return false
  }

  companion object {
    @JvmStatic
    fun getInstance(): AutoStartPermissionHelper {
      return AutoStartPermissionHelper()
    }

  }
}